@@ -247,6 +247,7 @@ class Horde_ActiveSync
     const CLASS_CALENDAR                        = 'Calendar';
     const CLASS_TASKS                           = 'Tasks';
     const CLASS_NOTES                           = 'Notes';
+    const CLASS_SMS                             = 'SMS';
 
     /* Filtertype constants */
     const FILTERTYPE_ALL                        = 0;
@@ -266,9 +267,9 @@ class Horde_ActiveSync
     const FOLDER_ROOT                           = 0;
 
     const VERSION_TWOFIVE                       = '2.5';
-    const VERSION_TWELVE                        = '12';
+    const VERSION_TWELVE                        = '12.0';
     const VERSION_TWELVEONE                     = '12.1';
-    const VERSION_FOURTEEN                      = '14';
+    const VERSION_FOURTEEN                      = '14.0';
     const VERSION_FOURTEENONE                   = '14.1';
 
     const MIME_SUPPORT_NONE                     = 0;
@@ -290,6 +291,8 @@ class Horde_ActiveSync
     const AUTH_REASON_USER_DENIED               = 'user';
     const AUTH_REASON_DEVICE_DENIED             = 'device';
 
+    const LIBRARY_VERSION                       = '2.5.6';
+
     /**
      * Logger
      *
@@ -829,6 +832,7 @@ class Horde_ActiveSync
         // clients may allow changing the protocol version.
         $device->version = $version;
         $this->_device = $device;
+
         // Don't bother with everything else if all we want are Options
         if ($cmd == 'Options') {
             $this->_doOptionsRequest();
@@ -851,6 +855,15 @@ class Horde_ActiveSync
         // We must send the eas header here, since some requests may start
         // output and be large enough to flush the buffer (e.g., GetAttachment)
         $this->activeSyncHeader();
+        if ($cmd != 'GetAttachment') {
+            $this->contentTypeHeader();
+        }
+
+        // Should we announce a new version is available to the client?
+        if ($device->version < $this->_maxVersion) {
+            header("X-MS-RP: ". $this->getSupportedVersions());
+        }
+
         $class = 'Horde_ActiveSync_Request_' . basename($cmd);
         if (class_exists($class)) {
             $request = new $class($this);
@@ -871,6 +884,10 @@ class Horde_ActiveSync
      */
     public function activeSyncHeader()
     {
+        header('Allow: OPTIONS,POST');
+        header('Server: Horde_ActiveSync Library v' . self::LIBRARY_VERSION);
+        header('Public: OPTIONS,POST');
+
         switch ($this->_maxVersion) {
         case self::VERSION_TWOFIVE:
             header('MS-Server-ActiveSync: 6.5.7638.1');
@@ -885,7 +902,7 @@ class Horde_ActiveSync
             header('MS-Server-ActiveSync: 14.0');
             break;
         case self::VERSION_FOURTEENONE:
-            header('MS-Server-ActiveSync: 14.1');
+            header('MS-Server-ActiveSync: 14.2');
         }
     }
 
@@ -934,7 +951,7 @@ class Horde_ActiveSync
         case self::VERSION_TWELVEONE:
         case self::VERSION_FOURTEEN:
         case self::VERSION_FOURTEENONE:
-            return 'Sync,SendMail,SmartForward,SmartReply,GetAttachment,GetHierarchy,CreateCollection,DeleteCollection,MoveCollection,FolderSync,FolderCreate,FolderDelete,FolderUpdate,MoveItems,GetItemEstimate,MeetingResponse,ResolveRecipients,ValidateCert,Provision,Settings,Search,Ping,ItemOperations';
+            return 'Sync,SendMail,SmartForward,SmartReply,GetAttachment,GetHierarchy,CreateCollection,DeleteCollection,MoveCollection,FolderSync,FolderCreate,FolderDelete,FolderUpdate,MoveItems,GetItemEstimate,MeetingResponse,Search,Settings,Ping,ItemOperations,Provision,ResolveRecipients,ValidateCert';
         }
     }
 
@@ -969,9 +986,9 @@ class Horde_ActiveSync
     }
 
     /**
-     * Obtain the ActiveSync protocol version
+     * Obtain the ActiveSync protocol version requested by the client headers.
      *
-     * @return string
+     * @return long
      */
     public function getProtocolVersion()
     {
@@ -1075,6 +1092,23 @@ class Horde_ActiveSync
         return $this->_globalError;
     }
 
+    /**
+     * Send the content type header.
+     *
+     */
+    public function contentTypeHeader($content_type = null)
+    {
+        if (!empty($content_type)) {
+            header('Content-Type: ' . $content_type);
+            return;
+        }
+        if ($this->_multipart) {
+            header('Content-Type: application/vnd.ms-sync.multipart');
+        } else {
+            header('Content-Type: application/vnd.ms-sync.wbxml');
+        }
+    }
+
     /**
      * Send the OPTIONS request response headers.
      *
@@ -1087,9 +1121,13 @@ class Horde_ActiveSync
     }
 
     /**
+     * Return the number of bytes corresponding to the requested trunction
+     * constant.
      *
-     * @param $truncation
-     * @return unknown_type
+     * @param integer $truncation  The constant.
+     *
+     * @return integer|boolean  Either the size, in bytes, to truncate or
+     *                          falso if no truncation.
      */
     static public function getTruncSize($truncation)
     {
@@ -1112,7 +1150,7 @@ class Horde_ActiveSync
             return 102400;
         case Horde_ActiveSync::TRUNCATION_8:
         case Horde_ActiveSync::TRUNCATION_NONE:
-            return 1048576; // We'll limit to 1MB anyway
+            return false;
         default:
             return 1024; // Default to 1Kb
         }
