@@ -42,6 +42,8 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
     const STATUS_KEYMISM                = 3;
     const STATUS_PROTERROR              = 4;
     const STATUS_SERVERERROR            = 5;
+    const STATUS_INVALID                = 6;
+    const STATUS_CONFLICT               = 7;
     const STATUS_NOTFOUND               = 8;
 
     // 12.1
@@ -338,9 +340,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             $this->_activeSync,
             $this->_encoder);
 
+        $cnt_global = 0;
         foreach ($this->_collections as $id => $collection) {
             $statusCode = self::STATUS_SUCCESS;
             $changecount = 0;
+            $cnt_collection = 0;
             try {
                 $this->_collections->initCollectionState($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
@@ -359,7 +363,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 return false;
             }
 
-            // HACK!!
             // Outlook explicitly tells the server to NOT check for server
             // changes when importing client changes, unlike EVERY OTHER client
             // out there. This completely screws up many things like conflict
@@ -485,15 +488,43 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     // Output server IDs for new items we received and added from PIM
                     if (!empty($collection['clientids'])) {
                         foreach ($collection['clientids'] as $clientid => $serverid) {
+                            if ($serverid) {
+                                $status = self::STATUS_SUCCESS;
+                            } else {
+                                $status = self::STATUS_INVALID;
+                            }
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_ADD);
+                            // If we have clientids and a CLASS_EMAIL, this is
+                            // a SMS response.
+                            if ($collection['class'] == Horde_ActiveSync::CLASS_EMAIL) {
+                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
+                                $this->_encoder->content(Horde_ActiveSync::CLASS_SMS);
+                                $this->_encoder->endTag();
+                            }
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_CLIENTENTRYID);
                             $this->_encoder->content($clientid);
                             $this->_encoder->endTag();
+                            if ($status == self::STATUS_SUCCESS) {
+                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
+                                $this->_encoder->content($serverid);
+                                $this->_encoder->endTag();
+                            }
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
+                            $this->_encoder->content($status);
+                            $this->_encoder->endTag();
+                            $this->_encoder->endTag();
+                        }
+                    }
+
+                    // Output any SYNC_CHANGE failures
+                    if (!empty($collection['importfailures'])) {
+                        foreach ($collection['importfailures'] as $id => $reason) {
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CHANGE);
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                            $this->_encoder->content($serverid);
+                            $tihs->_encoder->content($id);
                             $this->_encoder->endTag();
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content(self::STATUS_SUCCESS);
+                            $this->_encoder->content($reason);
                             $this->_encoder->endTag();
                             $this->_encoder->endTag();
                         }
@@ -542,14 +573,27 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     (!empty($collection['getchanges']) ||
                      (!isset($collection['getchanges']) && !empty($collection['synckey'])))) {
 
-                    if (!empty($collection['windowsize']) && !empty($changecount) && $changecount > $collection['windowsize']) {
+                    if ((!empty($changecount) && $changecount > $collection['windowsize']) ||
+                        ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
+
+                        $this->_logger->info(sprintf(
+                            '[%s] Sending MOREAVAILABLE. $changecount = %d, $cnt_global = %d',
+                            $this->_procid, $changecount, $cnt_global));
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
                     }
 
                     if (!empty($changecount)) {
                         $exporter->setChanges($this->_collections->getCollectionChanges(), $collection);
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_COMMANDS);
-                        while ($progress = $exporter->sendNextChange()) { }
+                        $cnt_collection = 0;
+                        while ($cnt_collection < $collection['windowsize'] &&
+                               $cnt_global < $this->_collections->getDefaultWindowSize() &&
+                               $progress = $exporter->sendNextChange()) {
+
+                            ++$cnt_collection;
+                            ++$cnt_global;
+                        }
+
                         $this->_encoder->endTag();
                     }
                 }
@@ -651,15 +695,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         $this->_handleError($collection);
                         return false;
                     }
-                    $this->_logger->info(sprintf(
-                        '[%s] Requesting WINDOWSIZE of %s',
-                        $this->_procid,
-                        $collection['windowsize']));
                     if ($collection['windowsize'] < 1 || $collection['windowsize'] > self::MAX_WINDOW_SIZE) {
                         $this->_logger->err(sprintf(
-                            '[%s] Bad windowsize sent, defaulting to 100',
+                            '[%s] Bad windowsize sent, defaulting to 512',
                             $this->_procid));
-                        $collection['windowsize'] = 100;
+                        $collection['windowsize'] = self::MAX_WINDOW_SIZE;
                     }
                     break;
 
@@ -694,6 +734,12 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         if (empty($this->_device->supported)) {
                             $this->_device->supported = array();
                         }
+                        // Not all clients send the $collection['class'] in more
+                        // recent EAS versions. Grab it from the collection
+                        // handler if needed.
+                        if (empty($collection['class'])) {
+                            $collection['class'] = $this->_collections->getCollectionClass($collection['id']);
+                        }
                         $this->_device->supported[$collection['class']] = $collection['supported'];
                         $this->_device->save();
                     }
@@ -825,7 +871,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_SERVERENTRYID)) {
 
                 $serverid = $this->_decoder->getElementContent();
-                if (!$this->_decoder->getElementEndTag()) {
+                // Work around broken clients (Blackberry) that can send empty
+                // $serverid values as a single empty <SYNC_SERVERENTRYID /> tag.
+                if ($serverid !== false && !$this->_decoder->getElementEndTag()) {
                     $this->_statusCode = self::STATUS_PROTERROR;
                     $this->_handleGlobalSyncError();
                     $this->_logger->err('Parsing Error - expecting </SYNC_SERVERENTRYID>');
@@ -836,11 +884,12 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             }
 
             // This tag is only sent here during > 12.1 and SYNC_ADD requests...
-            // and it's not event sent by all clients. Parse it if it's there,
+            // and it's not even sent by all clients. Parse it if it's there,
             // ignore it if not.
             if ($this->_activeSync->device->version > Horde_ActiveSync::VERSION_TWELVEONE &&
                 $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_ADD &&
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_FOLDERTYPE)) {
+
                 $collection['class'] = $this->_decoder->getElementContent();
                 if (!$this->_decoder->getElementEndTag()) {
                     $this->_statusCode = self::STATUS_PROTERROR;
@@ -890,7 +939,12 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $appdata = Horde_ActiveSync::messageFactory('Note');
                     $appdata->decodeStream($this->_decoder);
                     break;
+                case Horde_ActiveSync::CLASS_SMS:
+                    $appdata = Horde_ActiveSync::messageFactory('Mail');
+                    $appdata->decodeStream($this->_decoder);
+                    break;
                 }
+
                 if (!$this->_decoder->getElementEndTag()) {
                     // End application data
                     $this->_statusCode = self::STATUS_PROTERROR;
@@ -903,25 +957,34 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 switch ($element[Horde_ActiveSync_Wbxml::EN_TAG]) {
                 case Horde_ActiveSync::SYNC_MODIFY:
                     if (isset($appdata)) {
-                        $importer->importMessageChange(
+                        $id = $importer->importMessageChange(
                             $serverid, $appdata, $this->_device, false);
-                        $collection['importedchanges'] = true;
+                        if ($id && !is_array($id)) {
+                            $collection['importedchanges'] = true;
+                        } elseif (is_array($id)) {
+                            $collection['importfailures'][$id[0]] = $id[1];
+                        }
                     }
                     break;
 
                 case Horde_ActiveSync::SYNC_ADD:
                     if (isset($appdata)) {
                         $id = $importer->importMessageChange(
-                            false, $appdata, $this->_device, $clientid);
+                            false, $appdata, $this->_device, $clientid, $collection['class']);
                         if ($clientid && $id) {
                             $collection['clientids'][$clientid] = $id;
                             $collection['importedchanges'] = true;
+                        } elseif (!$id) {
+                            $collection['clientids'][$clientid] = false;
                         }
                     }
                     break;
 
                 case Horde_ActiveSync::SYNC_REMOVE:
-                    $collection['removes'][] = $serverid;
+                    // Work around broken clients that send empty $serverid.
+                    if ($serverid) {
+                        $collection['removes'][] = $serverid;
+                    }
                     break;
 
                 case Horde_ActiveSync::SYNC_FETCH:
@@ -945,7 +1008,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $results = $importer->importMessageMove($collection['removes'], $folderid);
             } else {
                 $results = $importer->importMessageDeletion($collection['removes'], $collection['class']);
-                if (is_array($results['results'])) {
+                if (is_array($results)) {
                     $results['results'] = $results;
                     $results['missing'] = array_diff($collection['removes'], $results['results']);
                 }
