@@ -20,7 +20,7 @@
  *
  * Needs a number of SQL tables present:
  *    syncStateTable (horde_activesync_state):
- *        sync_time:    - The timestamp of last sync
+ *        sync_timestamp:    - The timestamp of last sync
  *        sync_key:     - The syncKey for the last sync
  *        sync_pending: - If the last sync resulted in a MOREAVAILABLE, this
  *                        contains a list of UIDs that still need to be sent to
@@ -31,7 +31,8 @@
  *                        collections during a SYNC).
  *        sync_devid:   - The device id.
  *        sync_folderid:- The folder id for this sync.
- *        sync_user:    - The user for this synckey
+ *        sync_user:    - The user for this synckey.
+ *        sync_mod:     - The last modification stamp.
  *
  *    syncMapTable (horde_activesync_map):
  *        message_uid    - The server uid for the object
@@ -269,7 +270,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
 
         // Load the previous syncState from storage
         try {
-            $results = $this->_db->selectOne('SELECT sync_data, sync_devid, sync_time, sync_pending FROM '
+            $results = $this->_db->selectOne('SELECT sync_data, sync_devid, sync_mod, sync_pending FROM '
                 . $this->_syncStateTable . ' WHERE sync_key = ?', array($syncKey));
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err('Error in loading state from DB: ' . $e->getMessage());
@@ -298,8 +299,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     protected function _loadStateFromResults($results, $type = Horde_ActiveSync::REQUEST_TYPE_SYNC)
     {
         // Load the last known sync time for this collection
-        $this->_lastSyncStamp = !empty($results['sync_time'])
-            ? $results['sync_time']
+        $this->_lastSyncStamp = !empty($results['sync_mod'])
+            ? $results['sync_mod']
             : 0;
 
         // Pre-Populate the current sync timestamp in case this is only a
@@ -377,8 +378,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     {
         // Update state table to remember this last synctime and key
         $sql = 'INSERT INTO ' . $this->_syncStateTable
-            . ' (sync_key, sync_data, sync_devid, sync_time, sync_folderid, sync_user, sync_pending)'
-            . ' VALUES (?, ?, ?, ?, ?, ?, ?)';
+            . ' (sync_key, sync_data, sync_devid, sync_mod, sync_folderid, sync_user, sync_pending, sync_timestamp)'
+            . ' VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
 
         // Prepare state and pending data
         if ($this->_type == Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC) {
@@ -401,7 +402,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             (self::getSyncKeyCounter($this->_syncKey) == 1 ? 0 : $this->_thisSyncStamp),
             (!empty($this->_collection['id']) ? $this->_collection['id'] : Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC),
             $this->_deviceInfo->user,
-            $pending);
+            $pending,
+            time());
         $this->_logger->info(
             sprintf('[%s] Saving state: %s',
                 $this->_procid,
@@ -413,7 +415,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                         $params[3],
                         $params[4],
                         $params[5],
-                        count($this->_changes)),
+                        count($this->_changes),
+                        time()),
                     true)
                 )
             );
@@ -445,11 +448,12 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      *                          foldersync)
      * @param array $change     A stat/change hash describing the change.
      *  Contains:
-     *    - id: (mixed)       The message uid the change applies to.
+     *    - id: (mixed)         The message uid the change applies to.
      *    - serverid: (string)  The backend server id for the folder.
-     *    - parent: (string)  The parent of the message, normally the folder id.
-     *    - flags: (array)    If this is a flag change, the state of the flags.
-     *    - mod: (integer)    The modtime of this change.
+     *    - folderuid: (string) The EAS folder UID for the folder.
+     *    - parent: (string)    The parent of the current folder, if any.
+     *    - flags: (array)      If this is a flag change, the state of the flags.
+     *    - mod: (integer)      The modtime of this change.
      *
      * @param integer $origin   Flag to indicate the origin of the change:
      *    Horde_ActiveSync::CHANGE_ORIGIN_NA  - Not applicapble/not important
@@ -480,6 +484,15 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                 return;
             }
 
+            // Some requests like e.g., MOVEITEMS do not include the state
+            // information since there is no SYNCKEY. Attempt to map this from
+            // the $change array.
+            if (empty($this->_collection)) {
+                $this->_collection = array(
+                    'class' => $change['class'],
+                    'id' => $change['folderuid']);
+            }
+
             // This is an incoming change from the PIM, store it so we
             // don't mirror it back to device.
             switch ($this->_collection['class']) {
@@ -523,16 +536,17 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             default:
                 $sql = 'INSERT INTO ' . $this->_syncMapTable
                     . ' (message_uid, sync_modtime, sync_key, sync_devid,'
-                    . ' sync_folderid, sync_user, sync_clientid)'
-                    . ' VALUES (?, ?, ?, ?, ?, ?, ?)';
+                    . ' sync_folderid, sync_user, sync_clientid, sync_deleted)'
+                    . ' VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
                 $params = array(
                    $change['id'],
                    $change['mod'],
                    empty($this->_syncKey) ? 0 : $this->_syncKey,
                    $this->_deviceInfo->id,
-                   $change['parent'],
+                   $change['serverid'],
                    $user,
-                   $clientid);
+                   $clientid,
+                   $type == Horde_ActiveSync::CHANGE_TYPE_DELETE);
             }
 
             try {
@@ -814,15 +828,21 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     public function getLastSyncTimestamp($id = null, $user = null)
     {
-        if (empty($this->_deviceInfo) && empty($id) && empty($user)) {
+        if (empty($id) && empty($this->_deviceInfo)) {
             throw new Horde_ActiveSync_Exception('Device not loaded.');
         }
         $id = empty($id) ? $this->_deviceInfo->id : $id;
         $user = empty($user) ? $this->_deviceInfo->user : $user;
+        $params = array($id);
+
+        $sql = 'SELECT MAX(sync_timestamp) FROM ' . $this->_syncStateTable . ' WHERE sync_devid = ?';
+        if (!empty($user)) {
+            $sql .= ' AND sync_user = ?';
+            $params[] = $user;
+        }
 
-        $sql = 'SELECT MAX(sync_time) FROM ' . $this->_syncStateTable . ' WHERE sync_devid = ? AND sync_user = ?';
         try {
-            return $this->_db->selectValue($sql, array($id, $user));
+            return $this->_db->selectValue($sql, $params);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -841,11 +861,6 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     public function getChanges(array $options = array())
     {
-        $this->_thisSyncStamp = $this->_backend->getSyncStamp(empty($this->_collection['id']) ? null : $this->_collection['id'], $this->_lastSyncStamp);
-        if (!$this->_thisSyncStamp) {
-            throw new Horde_ActiveSync_Exception_StaleState('Detecting a change in timestamp or modification sequence. Reseting state.');
-        }
-
         if (!empty($this->_collection['id'])) {
             // How far back to sync (for those collections that use this)
             $cutoffdate = self::_getCutOffDate(!empty($this->_collection['filtertype'])
@@ -867,6 +882,21 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                     return $this->_changes;
                 }
 
+                // Get the current syncStamp from the backend.
+                $this->_thisSyncStamp = $this->_backend->getSyncStamp(
+                    empty($this->_collection['id']) ? null : $this->_collection['id'],
+                    $this->_lastSyncStamp);
+                if ($this->_thisSyncStamp === false) {
+                    throw new Horde_ActiveSync_Exception_StaleState(
+                        'Detecting a change in timestamp or modification sequence. Reseting state.');
+                }
+
+                $this->_logger->info(sprintf(
+                    '[%s] Using MODSEQ %s for %s.',
+                    $this->_procid,
+                    $this->_thisSyncStamp,
+                    $this->_collection['id']));
+
                 // No existing changes, poll the backend
                 $changes = $this->_backend->getServerChanges(
                     $this->_folder,
@@ -926,13 +956,24 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                     break;
                 default:
                     foreach ($changes as $change) {
-                        $stat = $this->_backend->statMessage($this->_folder->serverid(), $change['id']);
-                        $ts = $this->_getPIMChangeTS($change['id']);
-                        if ($ts && $ts >= $stat['mod']) {
+                        $pim_ts = $this->_getPIMChangeTS($change['id'], $change['type']);
+                        if ($pim_ts && $change['type'] == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
+                            // If we have a delete, don't bother stating the message,
+                            // If we have a delete entry in the map table, the
+                            // entry should already be deleted on the client, we
+                            // should never, ever need to send a REMOVE to the client
+                            // if we have a delete entry in the map table.
+                            $stat['mod'] = 0;
+                        } else {
+                            // stat only returns MODIFY times, not deletion times,
+                            // so will return (int)0 for ADD or DELETE.
+                            $stat = $this->_backend->statMessage($this->_folder->serverid(), $change['id']);
+                        }
+                        if ($pim_ts && $pim_ts >= $stat['mod']) {
                             $this->_logger->info(sprintf(
                                 '[%s] Ignoring PIM initiated change for %s (PIM TS: %s Stat TS: %s)',
                                 $this->_procid,
-                                $change['id'], $ts, $stat['mod']));
+                                $change['id'], $pim_ts, $stat['mod']));
                         } else {
                             $this->_changes[] = $change;
                         }
@@ -1378,19 +1419,29 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      * @return integer|null The timestamp of the last PIM-initiated change for
      *                      the specified uid, or null if none found.
      */
-    protected function _getPIMChangeTS($uid)
+    protected function _getPIMChangeTS($uid, $type)
     {
         $sql = 'SELECT MAX(sync_modtime) FROM ' . $this->_syncMapTable
-            . ' WHERE message_uid = ? AND sync_devid = ? AND sync_user = ? AND'
-            . ' sync_key IN (?, ?)'
-            . ' GROUP BY message_uid';
+            . ' WHERE message_uid = ? AND sync_devid = ? AND sync_user = ?';
+
+        if ($type == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
+            $sql .= ' AND sync_deleted = ?';
+        }
+
+        $sql .= ' AND sync_key IN (?, ?) GROUP BY message_uid';
 
         // Get the allowed synckeys to include.
         $uuid = self::getSyncKeyUid($this->_syncKey);
         $cnt = self::getSyncKeyCounter($this->_syncKey);
+        $values = array($uid, $this->_deviceInfo->id, $this->_deviceInfo->user);
+        if ($type == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
+            $values[] = true;
+        }
+        foreach (array($uuid . $cnt, $uuid . ($cnt - 1)) as $v) {
+            $values[] = $v;
+        }
         try {
-            return $this->_db->selectValue(
-                $sql, array($uid, $this->_deviceInfo->id, $this->_deviceInfo->user, $uuid . $cnt, $uuid . ($cnt - 1)));
+            return $this->_db->selectValue($sql, $values);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -1425,26 +1476,6 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
     }
 
-    /**
-     * Get the timestamp for the last successful sync for the current collection
-     *
-     * @return integer  The timestamp of the last successful sync or 0 if none
-     */
-    protected function _getLastSyncTS()
-    {
-        $sql = 'SELECT MAX(sync_time) FROM ' . $this->_syncStateTable
-            . ' WHERE sync_folderid = ? AND sync_devid = ?';
-
-        try {
-            $this->_lastSyncStamp = $this->_db->selectValue(
-                $sql, array($this->_collection['id'], $this->_deviceInfo->id));
-        } catch (Horde_Db_Exception $e) {
-            throw new Horde_ActiveSync_Exception($e);
-        }
-
-        return !empty($this->_lastSyncStamp) ? $this->_lastSyncStamp : 0;
-    }
-
     /**
      * Determines if a specific email change originated from the client. Used to
      * avoid mirroring back client initiated changes.
